# copyright (C) 1997-2000 Jean-Luc Fontaine (mailto:jfontain@free.fr)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: scrollbl.tcl,v 1.2.1.1 2000/02/19 15:27:17 jfontain Exp $}


class scrollingLabel {

    proc scrollingLabel {this parentPath args} composite {[new frame $parentPath] $args} {
        set path $widget::($this,path)
        composite::manage $this\
            [new label $path -font $widget::(default,ButtonFont)] 0 [new label $path -font $widget::(default,ButtonFont)] 1
        composite::complete $this
        bind $path <Configure> "scrollingLabel::refresh $this %w"                     ;# now that everything is initialized properly
    }

    proc ~scrollingLabel {this} {}

    proc options {this} {                                                                               ;# force text initialization
        return [list\
            [list -font font Font $widget::(default,ButtonFont) $widget::(default,ButtonFont)]\
            [list -interval interval Interval 10 10]\
            [list -step step Step 1 1]\
            [list -text text Text {}]\
        ]
    }

    proc set-interval {this value} {}
    proc set-step {this value} {}

    proc set-font {this value} {
        $composite::($this,0,path) configure -font $value
        $composite::($this,1,path) configure -font $value
        set scrollingLabel::($this,textWidth) [winfo reqwidth $composite::($this,0,path)]
        refresh $this [winfo width $widget::($this,path)]                                                           ;# force refresh
    }

    proc set-text {this value} {
        $composite::($this,0,path) configure -text $value
        set scrollingLabel::($this,textWidth) [winfo reqwidth $composite::($this,0,path)]
        refresh $this [winfo width $widget::($this,path)]                                                           ;# force refresh
    }

    proc refresh {this width} {
        $widget::($this,path) configure -height [winfo reqheight $composite::($this,0,path)]
        place $composite::($this,0,path) -anchor nw -x 0 -y 0
        set scrollingLabel::($this,x) 0
        set scrollingLabel::($this,left) 0
        set scrollingLabel::($this,right) 1
        set text $composite::($this,-text)
        if {$width<$scrollingLabel::($this,textWidth)} {                                                       ;# scrolling required
            append text { ... }                                                                      ;# add separator between frames
            $composite::($this,0,path) configure -text $text
            set scrollingLabel::($this,span) [winfo reqwidth $composite::($this,0,path)]
            $composite::($this,1,path) configure -text $text
            place $composite::($this,1,path) -anchor nw -x $scrollingLabel::($this,span) -y 0
            scroll $this
        } else {                                                                       ;# text fits in window, no scrolling required
            catch {unset scrollingLabel::($this,span)}
            $composite::($this,0,path) configure -text $text
            place forget $composite::($this,1,path)                                                                      ;# unneeded
            $composite::($this,1,path) configure -text {}
        }
    }

    proc scroll {this} {
        if {![info exists scrollingLabel::($this,span)]} return                   ;# window may have been resized so everything fits
        if {$scrollingLabel::($this,x)<=-$scrollingLabel::($this,span)} {   ;# left frame right edge is about to disappear from view
            set value $scrollingLabel::($this,left)                                                                   ;# swap frames
            set scrollingLabel::($this,left) $scrollingLabel::($this,right)
            set scrollingLabel::($this,right) $value
            set scrollingLabel::($this,x) 0
        }
        place $composite::($this,$scrollingLabel::($this,left),path) -x $scrollingLabel::($this,x)
        place $composite::($this,$scrollingLabel::($this,right),path)\
            -x [expr {$scrollingLabel::($this,x)+$scrollingLabel::($this,span)}]
        incr scrollingLabel::($this,x) -$composite::($this,-step)
        catch {after cancel $scrollingLabel::($this,event)}                                                 ;# avoid race conditions
        set scrollingLabel::($this,event) [after $composite::($this,-interval) "scrollingLabel::scroll $this"]
    }

}
